@@ -8,25 +8,25 @@ class PlywoodValue(object):
     def __init__(self, location):
         self.location = location
 
-    def get_attr(self, scope, right):
-        raise Exception("{self!r} has no property {right!r}".format(self=self, right=right))
+    def get_attr(self, attr, scope):
+        raise Exception("{self!r} has no property {attr!r}".format(self=self, attr=attr))
 
     def plywood_value(self):
         return self
 
     def python_value(self, scope):
-        raise NotImplemented
+        raise NotImplementedError
 
     def get_value(self, scope):
         return self
 
     def call(self, states, scope, arguments, block):
         if len(block.lines):
-            raise InvalidArguments('`str` does not support block argument')
+            raise InvalidArguments('{0!s} does not support block argument'.format(self))
         if len(arguments.args) != 1 \
             or len(arguments.kwargs):
-            raise InvalidArguments('`str` only accepts one conditional argument')
-        retval = self.python_value(scope) + arguments.args[0].python_value(scope)
+            raise InvalidArguments('{0!s} only accepts one conditional argument'.format(self))
+        retval = unicode(self.python_value(scope)) + unicode(arguments.args[0].python_value(scope))
         return [Continue()], PlywoodWrapper(self.location, retval)
 
     def run(self, states, scope):
@@ -61,6 +61,9 @@ class PlywoodBlock(PlywoodValue):
     def __str__(self):
         return '\n'.join(str(v) for v in self.lines)
 
+    def __unicode__(self):
+        return '\n'.join(unicode(v) for v in self.lines)
+
     def python_value(self, scope):
         states = [Continue()]
         return self.run(states, scope)[1].python_value(scope)
@@ -82,7 +85,7 @@ class PlywoodBlock(PlywoodValue):
                 else:
                     cmd_ret = cmd_ret.python_value(scope)
                     if cmd_ret is not None:
-                        retval += str(cmd_ret)
+                        retval += unicode(cmd_ret)
                         suppress_newline = SuppressNewline() in states or SuppressOneNewline() in states
                         if not self.inline and not suppress_newline:
                             retval += scope['__separator']
@@ -113,6 +116,12 @@ class PlywoodVariable(PlywoodValue):
     def get_name(self):
         return self.name
 
+    def set_attr(self, attr, value, scope):
+        scope[self.name] = value
+
+    def set_item(self, attr, value, scope):
+        scope[self.name] = value
+
     def python_value(self, scope):
         return self.get_value(scope).python_value(scope)
 
@@ -129,8 +138,8 @@ class PlywoodVariable(PlywoodValue):
             line_no, line = this_line(scope['__input'], self.location)
             raise PlywoodKeyError(self.name, scope, scope['__input'])
 
-    def get_attr(self, scope, right):
-        return self.get_value(scope).get_attr(scope, right)
+    def get_attr(self, attr, scope):
+        return self.get_value(scope).get_attr(attr, scope)
 
     def __eq__(self, other):
         return isinstance(other, PlywoodVariable) and other.name == self.name
@@ -152,21 +161,14 @@ class PlywoodString(PlywoodValue):
         self.value = value
         super(PlywoodString, self).__init__(location)
 
-    def call(self, states, scope, arguments, block):
-        if len(block.lines):
-            raise InvalidArguments('`str` does not support block argument')
-        if len(arguments.args) != 1 \
-            or len(arguments.kwargs):
-            raise InvalidArguments('`str` only accepts one conditional argument')
-        retval = unicode(self.python_value(scope)) + unicode(arguments.args[0].python_value(scope))
-        return [Continue()], PlywoodWrapper(self.location, retval)
-
     @staticmethod
     def unindent(value, return_lang=False):
         indent = None
         lines = value.splitlines()
-        if return_lang:
+        lang = ''
+        if return_lang and len(lines) > 1:
             lang = lines.pop(0)
+        # pop off the last line if it is empty
         if lines and lines[-1] == '':
             lines.pop()
 
@@ -187,7 +189,7 @@ class PlywoodString(PlywoodValue):
                 break
         if indent:
             lines = map(lambda line: line[len(indent):], lines)
-            value = "\n".join(lines)
+        value = "\n".join(lines)
         if return_lang:
             return lang, value
         return value
@@ -262,13 +264,32 @@ class PlywoodString(PlywoodValue):
 class PlywoodPythonValue(PlywoodValue):
     def __init__(self, location, value):
         self.value = value
+        if callable(value):
+            raise Exception('hey now')
         super(PlywoodPythonValue, self).__init__(location)
 
     def python_value(self, scope):
         return self.value
 
-    def get_attr(self, scope, right):
-        return getattr(self.python_value(scope), right.get_name())
+    def set_attr(self, attr, value, scope):
+        setattr(self.value, attr.get_name(), value.python_value(scope))
+
+    def get_attr(self, attr, scope):
+        val = self.python_value(scope)
+        attr = attr.get_name()
+        if hasattr(val, attr):
+            return getattr(val, attr)
+        return None
+
+    def set_item(self, attr, value, scope):
+        self.value[attr.get_name()] = value.python_value(scope)
+
+    def get_item(self, attr, scope):
+        val = self.python_value(scope)
+        key = attr.get_name()
+        if key in val:
+            return val[key]
+        return None
 
     def __repr__(self):
         return '{type.__name__}({self.value!r})'.format(type=type(self), self=self)
@@ -276,6 +297,9 @@ class PlywoodPythonValue(PlywoodValue):
     def __str__(self):
         return str(self.value)
 
+    def __unicode__(self):
+        return unicode(self.value)
+
 
 class PlywoodNumber(PlywoodValue):
     def __init__(self, location, value):
@@ -284,10 +308,10 @@ class PlywoodNumber(PlywoodValue):
 
     def call(self, states, scope, arguments, block):
         if len(block.lines):
-            raise InvalidArguments('`str` does not support block argument')
+            raise InvalidArguments('{0!s} does not support block argument'.format(self))
         if len(arguments.args) != 1 \
             or len(arguments.kwargs):
-            raise InvalidArguments('`str` only accepts one conditional argument')
+            raise InvalidArguments('{0!s} only accepts one conditional argument'.format(self))
         retval = self.python_value(scope) * arguments.args[0].python_value(scope)
         return [Continue()], PlywoodWrapper(self.location, retval)
 
@@ -300,6 +324,9 @@ class PlywoodNumber(PlywoodValue):
     def __str__(self):
         return str(self.value)
 
+    def __unicode__(self):
+        return unicode(self.value)
+
 
 class PlywoodOperator(PlywoodValue):
     INDENT = ''
@@ -344,8 +371,34 @@ class PlywoodOperator(PlywoodValue):
     def python_value(self, scope):
         return self.get_value(scope).python_value(scope)
 
-    def get_attr(self, scope, right):
-        return self.get_value(scope).get_attr(scope, right)
+    def get_attr(self, attr, scope):
+        return self.handle(self.operator, self.left, self.right, scope).get_attr(attr, scope)
+
+    def get_item(self, attr, scope):
+        target = self.handle(self.operator, self.left, self.right, scope)
+        return target.get_item(attr, scope)
+
+    def set_attr(self, attr, value, scope):
+        try:
+            handler, kwargs = self.OPERATORS[self.operator]
+        except KeyError:
+            raise Exception('No operator handler for {operator!r}'.format(operator=self.operator))
+        try:
+            setter = kwargs['setter']
+        except KeyError:
+            raise Exception('Operator {operator!r} cannot set values'.format(operator=self.operator))
+        return setter(self, attr, value, scope)
+
+    def set_item(self, attr, value, scope):
+        try:
+            handler, kwargs = self.OPERATORS[self.operator]
+        except KeyError:
+            raise Exception('No operator handler for {operator!r}'.format(operator=self.operator))
+        try:
+            setter = kwargs['setter']
+        except KeyError:
+            raise Exception('Operator {operator!r} cannot set values'.format(operator=self.operator))
+        return setter(self, attr, value, scope)
 
     def __repr__(self):
         indent = type(self).INDENT
@@ -362,6 +415,14 @@ class PlywoodOperator(PlywoodValue):
             op = ' ' + op + ' '
         return '{self.left}{op}{self.right}'.format(self=self, op=op)
 
+    def __unicode__(self):
+        op = unicode(self.operator)
+        if self.operator == '[]':
+            return '{self.left}[{self.right}]'.format(self=self)
+        if self.operator not in ['.']:
+            op = ' ' + op + ' '
+        return '{self.left}{op}{self.right}'.format(self=self, op=op)
+
 
 class PlywoodCallOperator(PlywoodOperator):
     def __init__(self, left, right, block=None):
@@ -397,6 +458,13 @@ class PlywoodCallOperator(PlywoodOperator):
             retval += ':\n{indent}    {block}'.format(self=self, block=str(self.block).replace("\n", "\n    " + indent), indent=indent)
         return retval
 
+    def __unicode__(self):
+        indent = ''
+        retval = '{indent}{self.left}{self.right}'.format(self=self, indent=indent)
+        if self.block:
+            retval += ':\n{indent}    {block}'.format(self=self, block=unicode(self.block).replace("\n", "\n    " + indent), indent=indent)
+        return retval
+
 
 class PlywoodUnaryOperator(PlywoodValue):
     OPERATORS = {}
@@ -476,6 +544,15 @@ class PlywoodParens(PlywoodValue):
             between = ', '
         return '(' + ', '.join(str(v) for v in self.args) + between + ', '.join(str(v) for v in self.kwargs) + extra + ')'
 
+    def __unicode__(self):
+        extra = ''
+        if self.is_set and len(self.args) == 1:
+            extra = ','
+        between = ''
+        if self.args and self.kwargs:
+            between = ', '
+        return '(' + ', '.join(unicode(v) for v in self.args) + between + ', '.join(unicode(v) for v in self.kwargs) + extra + ')'
+
 
 class PlywoodKvp(object):
     def __init__(self, key, value, kwarg=False):
@@ -499,6 +576,12 @@ class PlywoodKvp(object):
         else:
             return str(self.key) + self.separator + str(self.value)
 
+    def __unicode__(self):
+        if self.separator == '=':
+            return unicode(self.key)[1:-1] + self.separator + unicode(self.value)
+        else:
+            return unicode(self.key) + self.separator + unicode(self.value)
+
 
 class PlywoodList(PlywoodValue):
     def __init__(self, location, values, force_list=False):
@@ -518,6 +601,9 @@ class PlywoodList(PlywoodValue):
     def __str__(self):
         return '[' + ', '.join(str(v) for v in self.values) + ']'
 
+    def __unicode__(self):
+        return '[' + ', '.join(unicode(v) for v in self.values) + ']'
+
 
 class PlywoodXml(PlywoodValue):
     def __init__(self, location, element, arguments):
@@ -551,6 +637,9 @@ class PlywoodXml(PlywoodValue):
     def __str__(self):
         return '<' + self.element + ' ' + ', '.join(str(v) for v in self.arguments) + '>'
 
+    def __unicode__(self):
+        return '<' + self.element + ' ' + ', '.join(unicode(v) for v in self.arguments) + '>'
+
 
 class PlywoodIndices(PlywoodValue):
     def __init__(self, location, values, force_list):
@@ -558,6 +647,14 @@ class PlywoodIndices(PlywoodValue):
         self.force_list = force_list
         super(PlywoodIndices, self).__init__(location)
 
+    def python_value(self, scope):
+        if self.force_list:
+            return [value.python_value(scope) for value in self.values]
+        return self.values[0].python_value(scope)
+
+    def get_name(self):
+        return self.values[0].get_name()
+
     def __getitem__(self, key):
         return self.values.__getitem__(key)
 
@@ -567,6 +664,9 @@ class PlywoodIndices(PlywoodValue):
     def __str__(self):
         return ', '.join(str(v) for v in self.values) + (self.force_list and ',' or '')
 
+    def __unicode__(self):
+        return ', '.join(unicode(v) for v in self.values) + (self.force_list and ',' or '')
+
 
 class PlywoodSlice(PlywoodValue):
     def __init__(self, start, stop):
@@ -574,6 +674,9 @@ class PlywoodSlice(PlywoodValue):
         self.stop = stop
         super(PlywoodSlice, self).__init__(self.start.location)
 
+    def python_value(self, scope):
+        raise NotImplementedError
+
     def __repr__(self):
         return type(self).__name__ + '({self.start!r}:{self.stop!r})'.format(self=self)
 
@@ -589,33 +692,36 @@ class PlywoodDict(PlywoodValue):
     def python_value(self, scope):
         return dict((kvp.key.python_value(scope), kvp.value.python_value(scope)) for kvp in self.values)
 
-    def get_item(self, scope, right):
-        key = right.python_value(scope)
+    def set_item(self, attr, value, scope):
+        key = attr.python_value(scope)
+        for kvp in self.values:
+            if kvp.key.python_value(scope) == key:
+                kvp.value = value
+                return
+        raise KeyError(key)
+
+    def get_item(self, attr, scope):
+        key = attr.python_value(scope)
         for kvp in self.values:
             if kvp.key.python_value(scope) == key:
                 return kvp.value
         raise KeyError(key)
 
-    def get_attr(self, scope, right):
-        key = right.get_name()
+    def get_attr(self, attr, scope):
+        key = attr.get_name()
         for kvp in self.values:
             if kvp.key.python_value(scope) == key:
                 return kvp.value
         raise KeyError(key)
-        # if not isinstance(src, PlywoodValue):
-        #     key = right.get_name()
-        #     if hasattr(src, key):
-        #         return getattr(src, key)
-        #     else:
-        #         return src[key]
-        # else:
-        #     return src.get_attr(scope, right)
 
     def __repr__(self):
         return type(self).__name__ + '{' + ', '.join(repr(v) for v in self.values) + '}'
 
     def __str__(self):
-        return '{' + ', '.join(repr(v) for v in self.values) + '}'
+        return '{' + ', '.join(str(v) for v in self.values) + '}'
+
+    def __unicode__(self):
+        return '{' + ', '.join(unicode(v) for v in self.values) + '}'
 
 
 class PlywoodCallable(PlywoodValue):
@@ -685,9 +791,9 @@ class PlywoodHtmlPlugin(PlywoodCallable):
             copy.location = self.location
         return copy
 
-    def get_attr(self, scope, right):
+    def get_attr(self, attr, scope):
         copy = self.copy()
-        copy.classes.append(right.get_name())
+        copy.classes.append(attr.get_name())
         return copy
 
 
@@ -726,15 +832,6 @@ def PlywoodWrapper(location, value):
         return PlywoodString(location, value)
     if isinstance(value, int) or isinstance(value, long) or isinstance(value, float):
         return PlywoodNumber(location, value)
-    if isinstance(value, list):
-        return PlywoodList(location, value)
-    if isinstance(value, dict):
-        values = []
-        for key, value in value.iteritems():
-            key = PlywoodWrapper(location, key)
-            value = PlywoodWrapper(location, value)
-            values.append(PlywoodKvp(key, value))
-        return PlywoodDict(location, values)
     if callable(value):
         retval = PlywoodFunction(value)
         retval.location = location
