@@ -3,7 +3,7 @@ from . import assert_output
 
 def test_include():
     input = '''
-include 'test/include'
+include 'test/examples/include'
 '''
     desired = 'this is my test\n'
     assert_output(input, desired, {'word': 'test'})
@@ -11,7 +11,7 @@ include 'test/include'
 
 def test_include_scoped():
     input = '''
-include 'test/include', word='foo'
+include 'test/examples/include', word='foo'
 '''
     desired = 'this is my foo\n'
     assert_output(input, desired, {'word': 'test'})
